@@ -1,4 +1,4 @@
-VERSION = '2022.8.7'
+VERSION = '2023.2.8'
 
 SORTED_CLASSIFIERS = [
     'Development Status :: 1 - Planning',
@@ -40,6 +40,7 @@ SORTED_CLASSIFIERS = [
     'Environment :: GPU :: NVIDIA CUDA :: 10.0',
     'Environment :: GPU :: NVIDIA CUDA :: 10.1',
     'Environment :: GPU :: NVIDIA CUDA :: 10.2',
+    'Environment :: GPU :: NVIDIA CUDA :: 11',
     'Environment :: GPU :: NVIDIA CUDA :: 11.0',
     'Environment :: GPU :: NVIDIA CUDA :: 11.1',
     'Environment :: GPU :: NVIDIA CUDA :: 11.2',
@@ -48,6 +49,9 @@ SORTED_CLASSIFIERS = [
     'Environment :: GPU :: NVIDIA CUDA :: 11.5',
     'Environment :: GPU :: NVIDIA CUDA :: 11.6',
     'Environment :: GPU :: NVIDIA CUDA :: 11.7',
+    'Environment :: GPU :: NVIDIA CUDA :: 11.8',
+    'Environment :: GPU :: NVIDIA CUDA :: 12',
+    'Environment :: GPU :: NVIDIA CUDA :: 12 :: 12.0',
     "Environment :: Handhelds/PDA's",
     'Environment :: MacOS X',
     'Environment :: MacOS X :: Aqua',
@@ -61,6 +65,9 @@ SORTED_CLASSIFIERS = [
     'Environment :: Web Environment :: Buffet',
     'Environment :: Web Environment :: Mozilla',
     'Environment :: Web Environment :: ToscaWidgets',
+    'Environment :: WebAssembly',
+    'Environment :: WebAssembly :: Emscripten',
+    'Environment :: WebAssembly :: WASI',
     'Environment :: Win32 (MS Windows)',
     'Environment :: X11 Applications',
     'Environment :: X11 Applications :: GTK',
@@ -120,6 +127,9 @@ SORTED_CLASSIFIERS = [
     'Framework :: Django CMS :: 3.8',
     'Framework :: Django CMS :: 3.9',
     'Framework :: Django CMS :: 3.10',
+    'Framework :: Django CMS :: 3.11',
+    'Framework :: Django CMS :: 4.0',
+    'Framework :: Django CMS :: 4.1',
     'Framework :: FastAPI',
     'Framework :: Flake8',
     'Framework :: Flask',
@@ -151,6 +161,7 @@ SORTED_CLASSIFIERS = [
     'Framework :: Odoo :: 13.0',
     'Framework :: Odoo :: 14.0',
     'Framework :: Odoo :: 15.0',
+    'Framework :: Odoo :: 16.0',
     'Framework :: Opps',
     'Framework :: Paste',
     'Framework :: Pelican',
@@ -170,7 +181,10 @@ SORTED_CLASSIFIERS = [
     'Framework :: Plone :: 6.0',
     'Framework :: Plone :: Addon',
     'Framework :: Plone :: Core',
+    'Framework :: Plone :: Distribution',
     'Framework :: Plone :: Theme',
+    'Framework :: Pydantic',
+    'Framework :: Pydantic :: 1',
     'Framework :: Pylons',
     'Framework :: Pyramid',
     'Framework :: Pytest',
@@ -194,6 +208,7 @@ SORTED_CLASSIFIERS = [
     'Framework :: Wagtail :: 1',
     'Framework :: Wagtail :: 2',
     'Framework :: Wagtail :: 3',
+    'Framework :: Wagtail :: 4',
     'Framework :: ZODB',
     'Framework :: Zope',
     'Framework :: Zope2',
